@@ -1,20 +1,29 @@
 <?php
+// This file is part of Moodle - http://moodle.org/
+//
+// Moodle is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Moodle is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
 /**
- * @author Martin Dougiamas
- * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
- * @package moodle multiauth
- *
  * Authentication Plugin: IMAP Authentication
- *
  * Authenticates against an IMAP server.
  *
- * 2006-08-31  File created.
+ * @package auth_imap
+ * @author Martin Dougiamas
+ * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
  */
 
-if (!defined('MOODLE_INTERNAL')) {
-    die('Direct access to this script is forbidden.');    ///  It must be included from a Moodle page
-}
+defined('MOODLE_INTERNAL') || die();
 
 require_once($CFG->libdir.'/authlib.php');
 
@@ -46,7 +55,7 @@ class auth_plugin_imap extends auth_plugin_base {
         }
 
         global $CFG;
-        $hosts = split(';', $this->config->host);   // Could be multiple hosts
+        $hosts = explode(';', $this->config->host);   // Could be multiple hosts
 
         foreach ($hosts as $host) {                 // Try each host in turn
             $host = trim($host);
@@ -64,12 +73,16 @@ class auth_plugin_imap extends auth_plugin_base {
                     $host = '{'.$host.":{$this->config->port}/imap/tls}";
                 break;
 
+                case 'imapnosslcert':
+                    $host = '{'.$host.":{$this->config->port}/imap/novalidate-cert}";
+                break;
+
                 default:
                     $host = '{'.$host.":{$this->config->port}/imap}";
             }
 
             error_reporting(0);
-            $connection = imap_open($host, stripslashes($username), stripslashes($password), OP_HALFOPEN);
+            $connection = imap_open($host, $username, $password, OP_HALFOPEN);
             error_reporting($CFG->debug);
 
             if ($connection) {
@@ -108,10 +121,14 @@ class auth_plugin_imap extends auth_plugin_base {
      * Returns the URL for changing the user's pw, or empty if the default can
      * be used.
      *
-     * @return string
+     * @return moodle_url
      */
     function change_password_url() {
-        return $this->config->changepasswordurl;
+        if (!empty($this->config->changepasswordurl)) {
+            return new moodle_url($this->config->changepasswordurl);
+        } else {
+            return null;
+        }
     }
 
     /**
@@ -123,6 +140,8 @@ class auth_plugin_imap extends auth_plugin_base {
      * @param array $page An object containing all the data for this page.
      */
     function config_form($config, $err, $user_fields) {
+        global $OUTPUT;
+
         include "config.html";
     }
 
@@ -155,4 +174,4 @@ class auth_plugin_imap extends auth_plugin_base {
 
 }
 
-?>
+
