@@ -28,6 +28,7 @@ require_once('../config.php');
 require_once($CFG->dirroot.'/calendar/event_form.php');
 require_once($CFG->dirroot.'/calendar/lib.php');
 require_once($CFG->dirroot.'/course/lib.php');
+require_once($CFG->dirroot.'/calendar/renderer.php');
 
 $eventid = required_param('id', PARAM_INT);
 $confirm = optional_param('confirm', false, PARAM_BOOL);
@@ -54,7 +55,7 @@ if ($event->eventtype !== 'user' && $event->eventtype !== 'site') {
 $course = $DB->get_record('course', array('id'=>$courseid));
 require_login($course);
 if (!$course) {
-    $PAGE->set_context(get_context_instance(CONTEXT_SYSTEM)); //TODO: wrong
+    $PAGE->set_context(context_system::instance()); //TODO: wrong
 }
 
 // Check the user has the required capabilities to edit an event
@@ -68,8 +69,7 @@ $event->count_repeats();
 
 // Is used several times, and sometimes with modification if required
 $viewcalendarurl = new moodle_url(CALENDAR_URL.'view.php', array('view'=>'upcoming'));
-$viewcalendarurl->param('cal_y', userdate($event->timestart, '%Y'));
-$viewcalendarurl->param('cal_m', userdate($event->timestart, '%m'));
+$viewcalendarurl->param('time', $event->timestart, '%Y');
 
 // If confirm is set (PARAM_BOOL) then we have confirmation of initention to delete
 if ($confirm) {
