@@ -1,8 +1,8 @@
-<?php // $Id: latex.php,v 1.9.2.2 2010/04/10 00:17:45 iarenaza Exp $
+<?php
     // latex.php
     // render TeX stuff using latex - this will not work on all platforms
-    // or configurations. Only works on Linux and Mac with appropriate 
-    // software installed. 
+    // or configurations. Only works on Linux and Mac with appropriate
+    // software installed.
     // Much of this inspired/copied from Benjamin Zeiss' work
 
     class latex {
@@ -16,21 +16,15 @@
          * Other platforms could/should be added
          */
         function latex() {
-            global $CFG; 
-            
-            // construct directory structure
-            $this->temp_dir = $CFG->dataroot . "/temp/latex";
-            if (!file_exists("$CFG->dataroot/temp")) {
-                mkdir( "$CFG->dataroot/temp", $CFG->directorypermissions );
-            }
-            if (!file_exists( $this->temp_dir )) {
-                mkdir( $this->temp_dir, $CFG->directorypermissions );
-            }
+            global $CFG;
 
+            // construct directory structure
+            $this->temp_dir = $CFG->tempdir . "/latex";
+            make_temp_directory('latex');
         }
 
         /**
-         * Accessor function for support_platform field. 
+         * Accessor function for support_platform field.
          * @return boolean value of supported_platform
          */
         function supported() {
@@ -46,11 +40,11 @@
         function construct_latex_document( $formula, $fontsize=12 ) {
             global $CFG;
 
-            $formula = tex_sanitize_formula($formula);
+            $formula = filter_tex_sanitize_formula($formula);
 
             // $fontsize don't affects to formula's size. $density can change size
-            $doc =  "\\documentclass[{$fontsize}pt]{article}\n"; 
-            $doc .=  $CFG->filter_tex_latexpreamble;
+            $doc =  "\\documentclass[{$fontsize}pt]{article}\n";
+            $doc .= get_config('filter_tex', 'latexpreamble');
             $doc .= "\\pagestyle{empty}\n";
             $doc .= "\\begin{document}\n";
 //dlnsk            $doc .= "$ {$formula} $\n";
@@ -63,7 +57,7 @@
             return $doc;
         }
 
-        /** 
+        /**
          * execute an external command, with optional logging
          * @param string $command command to execute
          * @param file $log valid open file handle - log info will be written to this file
@@ -92,13 +86,15 @@
          * @return bool true if successful
          */
         function render( $formula, $filename, $fontsize=12, $density=240, $background='', $log=null ) {
-            
+
             global $CFG;
-  
+
             // quick check - will this work?
-            if (empty($CFG->filter_tex_pathlatex)) {
+            $pathlatex = get_config('filter_tex', 'pathlatex');
+            if (empty($pathlatex)) {
                 return false;
             }
+            $pathlatex = escapeshellarg(trim($pathlatex, " '\""));
 
             $doc = $this->construct_latex_document( $formula, $fontsize );
 
@@ -106,7 +102,8 @@
             $tex = "{$this->temp_dir}/$filename.tex";
             $dvi = "{$this->temp_dir}/$filename.dvi";
             $ps  = "{$this->temp_dir}/$filename.ps";
-            $img = "{$this->temp_dir}/$filename.{$CFG->filter_tex_convertformat}";
+            $convertformat = get_config('filter_tex', 'convertformat');
+            $img = "{$this->temp_dir}/$filename.{$convertformat}";
 
             // turn the latex doc into a .tex file in the temp area
             $fh = fopen( $tex, 'w' );
@@ -114,14 +111,15 @@
             fclose( $fh );
 
             // run latex on document
-            $command = "{$CFG->filter_tex_pathlatex} --interaction=nonstopmode $tex";
+            $command = "$pathlatex --interaction=nonstopmode --halt-on-error $tex";
             chdir( $this->temp_dir );
             if ($this->execute($command, $log)) { // It allways False on Windows
 //                return false;
-            } 
+            }
 
             // run dvips (.dvi to .ps)
-            $command = "{$CFG->filter_tex_pathdvips} -E $dvi -o $ps";
+            $pathdvips = escapeshellarg(trim(get_config('filter_tex', 'pathdvips'), " '\""));
+            $command = "$pathdvips -E $dvi -o $ps";
             if ($this->execute($command, $log )) {
                 return false;
             }
@@ -132,7 +130,8 @@
             } else {
                 $bg_opt = "";
             }
-            $command = "{$CFG->filter_tex_pathconvert} -density $density -trim $bg_opt $ps $img";
+            $pathconvert = escapeshellarg(trim(get_config('filter_tex', 'pathconvert'), " '\""));
+            $command = "$pathconvert -density $density -trim $bg_opt $ps $img";
             if ($this->execute($command, $log )) {
                 return false;
             }
@@ -151,7 +150,8 @@
             unlink( "{$this->temp_dir}/$filename.tex" );
             unlink( "{$this->temp_dir}/$filename.dvi" );
             unlink( "{$this->temp_dir}/$filename.ps" );
-            unlink( "{$this->temp_dir}/$filename.{$CFG->filter_tex_convertformat}" );
+            $convertformat = get_config('filter_tex', 'convertformat');
+            unlink( "{$this->temp_dir}/$filename.{$convertformat}" );
             unlink( "{$this->temp_dir}/$filename.aux" );
             unlink( "{$this->temp_dir}/$filename.log" );
             return;
@@ -160,4 +160,4 @@
     }
 
 
-?>
+
