@@ -67,14 +67,14 @@
  * To install one component:
  * <code>
  *     require_once($CFG->libdir.'/componentlib.class.php');
- *     if ($cd = new component_installer('http://download.moodle.org', 'langpack/2.0',
+ *     if ($cd = new component_installer('https://download.moodle.org', 'langpack/2.0',
  *                                       'es.zip', 'languages.md5', 'lang')) {
  *         $status = $cd->install(); //returns COMPONENT_(ERROR | UPTODATE | INSTALLED)
  *         switch ($status) {
  *             case COMPONENT_ERROR:
  *                 if ($cd->get_error() == 'remotedownloaderror') {
  *                     $a = new stdClass();
- *                     $a->url = 'http://download.moodle.org/langpack/2.0/es.zip';
+ *                     $a->url = 'https://download.moodle.org/langpack/2.0/es.zip';
  *                     $a->dest= $CFG->dataroot.'/lang';
  *                     print_error($cd->get_error(), 'error', '', $a);
  *                 } else {
@@ -220,12 +220,12 @@ class component_installer {
         $this->requisitesok = false;
 
     /// Check that everything we need is present
-        if (empty($this->sourcebase) || empty($this->zippath) || empty($this->zipfilename)) {
+        if (empty($this->sourcebase) || empty($this->zipfilename)) {
             $this->errorstring='missingrequiredfield';
             return false;
         }
     /// Check for correct sourcebase (this will be out in the future)
-        if ($this->sourcebase != 'http://download.moodle.org') {
+        if (!PHPUNIT_TEST and $this->sourcebase != 'https://download.moodle.org') {
             $this->errorstring='wrongsourcebase';
             return false;
         }
@@ -286,7 +286,12 @@ class component_installer {
              return COMPONENT_ERROR;
         }
     /// Download zip file and save it to temp
-        $source = $this->sourcebase.'/'.$this->zippath.'/'.$this->zipfilename;
+        if ($this->zippath) {
+            $source = $this->sourcebase.'/'.$this->zippath.'/'.$this->zipfilename;
+        } else {
+            $source = $this->sourcebase.'/'.$this->zipfilename;
+        }
+
         $zipfile= $CFG->tempdir.'/'.$this->zipfilename;
 
         if($contents = download_file_content($source)) {
@@ -474,7 +479,11 @@ class component_installer {
         $comp_arr = array();
 
     /// Define and retrieve the full md5 file
-        $source = $this->sourcebase.'/'.$this->zippath.'/'.$this->md5filename;
+        if ($this->zippath) {
+            $source = $this->sourcebase.'/'.$this->zippath.'/'.$this->md5filename;
+        } else {
+            $source = $this->sourcebase.'/'.$this->md5filename;
+        }
 
     /// Check if we have downloaded the md5 file before (per request cache)
         if (!empty($this->cachedmd5components[$source])) {
@@ -585,14 +594,13 @@ class lang_installer {
     /**
      * Prepare the installer
      *
-     * @todo Moodle major version is hardcoded here, should be obtained from version.php or so
      * @param string|array $langcode a code of the language to install
      */
     public function __construct($langcode = '') {
         global $CFG;
 
         $this->set_queue($langcode);
-        $this->version = '2.2';
+        $this->version = moodle_major_version(true);
 
         if (!empty($CFG->langotherroot) and $CFG->langotherroot !== $CFG->dataroot . '/lang') {
             debugging('The in-built language pack installer does not support alternative location ' .
@@ -666,9 +674,9 @@ class lang_installer {
     public function lang_pack_url($langcode = '') {
 
         if (empty($langcode)) {
-            return 'http://download.moodle.org/langpack/'.$this->version.'/';
+            return 'https://download.moodle.org/langpack/'.$this->version.'/';
         } else {
-            return 'http://download.moodle.org/download.php/langpack/'.$this->version.'/'.$langcode.'.zip';
+            return 'https://download.moodle.org/download.php/langpack/'.$this->version.'/'.$langcode.'.zip';
         }
     }
 
@@ -678,7 +686,7 @@ class lang_installer {
      * @return array|bool false if can not download
      */
     public function get_remote_list_of_languages() {
-        $source = 'http://download.moodle.org/langpack/' . $this->version . '/languages.md5';
+        $source = 'https://download.moodle.org/langpack/' . $this->version . '/languages.md5';
         $availablelangs = array();
 
         if ($content = download_file_content($source)) {
@@ -770,7 +778,7 @@ class lang_installer {
     protected function install_language_pack($langcode) {
 
         // initialise new component installer to process this language
-        $installer = new component_installer('http://download.moodle.org', 'download.php/direct/langpack/' . $this->version,
+        $installer = new component_installer('https://download.moodle.org', 'download.php/direct/langpack/' . $this->version,
             $langcode . '.zip', 'languages.md5', 'lang');
 
         if (!$installer->requisitesok) {
