@@ -2,12 +2,12 @@
 // Global M object is initilised in inline javascript
 
 /**
- * Add module to list of available modules that can be laoded from YUI.
+ * Add module to list of available modules that can be loaded from YUI.
  * @param {Array} modules
  */
 M.yui.add_module = function(modules) {
     for (var modname in modules) {
-        M.yui.loader.modules[modname] = modules[modname];
+        YUI_config.modules[modname] = modules[modname];
     }
 };
 /**
@@ -33,14 +33,22 @@ M.str = M.str || {};
  * @return {String}
  */
 M.util.image_url = function(imagename, component) {
-    var url = M.cfg.wwwroot + '/theme/image.php?theme=' + M.cfg.theme + '&image=' + imagename;
 
-    if (M.cfg.themerev > 0) {
-        url = url + '&rev=' + M.cfg.themerev;
+    if (!component || component == '' || component == 'moodle' || component == 'core') {
+        component = 'core';
     }
 
-    if (component && component != '' && component != 'moodle' && component != 'core') {
-        url = url + '&component=' + component;
+    var url = M.cfg.wwwroot + '/theme/image.php';
+    if (M.cfg.themerev > 0 && M.cfg.slasharguments == 1) {
+        if (!M.cfg.svgicons) {
+            url += '/_s';
+        }
+        url += '/' + M.cfg.theme + '/' + component + '/' + M.cfg.themerev + '/' + imagename;
+    } else {
+        url += '?theme=' + M.cfg.theme + '&component=' + component + '&rev=' + M.cfg.themerev + '&image=' + imagename;
+        if (!M.cfg.svgicons) {
+            url += '&svg=0';
+        }
     }
 
     return url;
@@ -102,9 +110,15 @@ M.util.CollapsibleRegion = function(Y, id, userpref, strtooltip) {
 
     // Get the height of the div at this point before we shrink it if required
     var height = this.div.get('offsetHeight');
+    var collapsedimage = 't/collapsed'; // ltr mode
+    if (right_to_left()) {
+        collapsedimage = 't/collapsed_rtl';
+    } else {
+        collapsedimage = 't/collapsed';
+    }
     if (this.div.hasClass('collapsed')) {
         // Add the correct image and record the YUI node created in the process
-        this.icon = Y.Node.create('<img src="'+M.util.image_url('t/collapsed', 'moodle')+'" alt="" />');
+        this.icon = Y.Node.create('<img src="'+M.util.image_url(collapsedimage, 'moodle')+'" alt="" />');
         // Shrink the div as it is collapsed by default
         this.div.setStyle('height', caption.get('offsetHeight')+'px');
     } else {
@@ -125,8 +139,14 @@ M.util.CollapsibleRegion = function(Y, id, userpref, strtooltip) {
     // Handler for the animation finishing.
     animation.on('end', function() {
         this.div.toggleClass('collapsed');
+        var collapsedimage = 't/collapsed'; // ltr mode
+        if (right_to_left()) {
+            collapsedimage = 't/collapsed_rtl';
+            } else {
+            collapsedimage = 't/collapsed';
+            }
         if (this.div.hasClass('collapsed')) {
-            this.icon.set('src', M.util.image_url('t/collapsed', 'moodle'));
+            this.icon.set('src', M.util.image_url(collapsedimage, 'moodle'));
         } else {
             this.icon.set('src', M.util.image_url('t/expanded', 'moodle'));
         }
@@ -181,7 +201,7 @@ M.util.CollapsibleRegion.prototype.icon = null;
  * @param String the value to set it to.
  */
 M.util.set_user_preference = function(name, value) {
-    YUI(M.yui.loader).use('io', function(Y) {
+    YUI().use('io', function(Y) {
         var url = M.cfg.wwwroot + '/lib/ajax/setuserpref.php?sesskey=' +
                 M.cfg.sesskey + '&pref=' + encodeURI(name) + '&value=' + encodeURI(value);
 
@@ -203,10 +223,16 @@ M.util.set_user_preference = function(name, value) {
 
 /**
  * Prints a confirmation dialog in the style of DOM.confirm().
- * @param object event A YUI DOM event or null if launched manually
- * @param string message The message to show in the dialog
- * @param string url The URL to forward to if YES is clicked. Disabled if fn is given
- * @param function fn A JS function to run if YES is clicked.
+ *
+ * @method show_confirm_dialog
+ * @param {EventFacade} e
+ * @param {Object} args
+ * @param {String} args.message The question to ask the user
+ * @param {Function} [args.callback] A callback to apply on confirmation.
+ * @param {Object} [args.scope] The scope to use when calling the callback.
+ * @param {Object} [args.callbackargs] Any arguments to pass to the callback.
+ * @param {String} [args.cancellabel] The label to use on the cancel button.
+ * @param {String} [args.continuelabel] The label to use on the continue button.
  */
 M.util.show_confirm_dialog = function(e, args) {
     var target = e.target;
@@ -214,47 +240,35 @@ M.util.show_confirm_dialog = function(e, args) {
         e.preventDefault();
     }
 
-    YUI(M.yui.loader).use('yui2-container', 'yui2-event', function(Y) {
-        var simpledialog = new YAHOO.widget.SimpleDialog('confirmdialog',
-            {width: '300px',
-              fixedcenter: true,
-              modal: true,
-              visible: false,
-              draggable: false
-            }
-        );
-
-        simpledialog.setHeader(M.str.admin.confirmation);
-        simpledialog.setBody(args.message);
-        simpledialog.cfg.setProperty('icon', YAHOO.widget.SimpleDialog.ICON_WARN);
-
-        var handle_cancel = function() {
-            simpledialog.hide();
-        };
-
-        var handle_yes = function() {
-            simpledialog.hide();
+    YUI().use('moodle-core-notification-confirm', function(Y) {
+        var confirmationDialogue = new M.core.confirm({
+            width: '300px',
+            center: true,
+            modal: true,
+            visible: false,
+            draggable: false,
+            title: M.util.get_string('confirmation', 'admin'),
+            noLabel: M.util.get_string('cancel', 'moodle'),
+            question: args.message
+        });
 
+        // The dialogue was submitted with a positive value indication.
+        confirmationDialogue.on('complete-yes', function(e) {
+            // Handle any callbacks.
             if (args.callback) {
-                // args comes from PHP, so callback will be a string, needs to be evaluated by JS
-                var callback = null;
-                if (Y.Lang.isFunction(args.callback)) {
-                    callback = args.callback;
-                } else {
-                    callback = eval('('+args.callback+')');
+                if (!Y.Lang.isFunction(args.callback)) {
+                    Y.log('Callbacks to show_confirm_dialog must now be functions. Please update your code to pass in a function instead.',
+                            'warn', 'M.util.show_confirm_dialog');
+                    return;
                 }
 
+                var scope = e.target;
                 if (Y.Lang.isObject(args.scope)) {
-                    var sc = args.scope;
-                } else {
-                    var sc = e.target;
+                    scope = args.scope;
                 }
 
-                if (args.callbackargs) {
-                    callback.apply(sc, args.callbackargs);
-                } else {
-                    callback.apply(sc);
-                }
+                var callbackargs = args.callbackargs || [];
+                args.callback.apply(scope, callbackargs);
                 return;
             }
 
@@ -268,9 +282,7 @@ M.util.show_confirm_dialog = function(e, args) {
                 window.location = targetancestor.get('href');
 
             } else if (target.test('input')) {
-                targetform = target.ancestor(function(node) { return node.get('tagName').toLowerCase() == 'form'; });
-                // We cannot use target.ancestor('form') on the previous line
-                // because of http://yuilibrary.com/projects/yui3/ticket/2531561
+                targetform = target.ancestor('form', true);
                 if (!targetform) {
                     return;
                 }
@@ -280,32 +292,26 @@ M.util.show_confirm_dialog = function(e, args) {
                 }
                 targetform.submit();
 
-            } else if (target.get('tagName').toLowerCase() == 'form') {
-                // We cannot use target.test('form') on the previous line because of
-                // http://yuilibrary.com/projects/yui3/ticket/2531561
+            } else if (target.test('form')) {
                 target.submit();
 
-            } else if (M.cfg.developerdebug) {
-                alert("Element of type " + target.get('tagName') + " is not supported by the M.util.show_confirm_dialog function. Use A, INPUT or FORM");
+            } else {
+                Y.log("Element of type " + target.get('tagName') +
+                        " is not supported by the M.util.show_confirm_dialog function. Use A, INPUT, or FORM",
+                        'warn', 'javascript-static');
             }
-        };
+        }, this);
 
-        if (!args.cancellabel) {
-            args.cancellabel = M.str.moodle.cancel;
-        }
-        if (!args.continuelabel) {
-            args.continuelabel = M.str.moodle.yes;
+        if (args.cancellabel) {
+            confirmationDialogue.set('noLabel', args.cancellabel);
         }
 
-        var buttons = [
-            {text: args.cancellabel,   handler: handle_cancel, isDefault: true},
-            {text: args.continuelabel, handler: handle_yes}
-        ];
+        if (args.continuelabel) {
+            confirmationDialogue.set('yesLabel', args.continuelabel);
+        }
 
-        simpledialog.cfg.queueProperty('buttons', buttons);
-
-        simpledialog.render(document.body);
-        simpledialog.show();
+        confirmationDialogue.render()
+                .show();
     });
 };
 
@@ -320,11 +326,20 @@ M.util.init_maximised_embed = function(Y, id) {
         if (Y.Lang.isString(el)) {
             el = Y.one('#' + el);
         }
-        var val = el.getStyle(prop);
-        if (val == 'auto') {
-            val = el.getComputedStyle(prop);
+        // Ensure element exists.
+        if (el) {
+            var val = el.getStyle(prop);
+            if (val == 'auto') {
+                val = el.getComputedStyle(prop);
+            }
+            val = parseInt(val);
+            if (isNaN(val)) {
+                return 0;
+            }
+            return val;
+        } else {
+            return 0;
         }
-        return parseInt(val);
     };
 
     var resize_object = function() {
@@ -340,7 +355,7 @@ M.util.init_maximised_embed = function(Y, id) {
 
         var headerheight = get_htmlelement_size('page-header', 'height');
         var footerheight = get_htmlelement_size('page-footer', 'height');
-        var newheight = parseInt(YAHOO.util.Dom.getViewportHeight()) - footerheight - headerheight - 100;
+        var newheight = parseInt(Y.one('body').get('docHeight')) - footerheight - headerheight - 100;
         if (newheight < 400) {
             newheight = 400;
         }
@@ -354,71 +369,6 @@ M.util.init_maximised_embed = function(Y, id) {
     };
 };
 
-/**
- * Attach handler to single_select
- */
-M.util.init_select_autosubmit = function(Y, formid, selectid, nothing) {
-    Y.use('event-key', function() {
-        var select = Y.one('#'+selectid);
-        if (select) {
-            // Try to get the form by id
-            var form = Y.one('#'+formid) || (function(){
-                // Hmmm the form's id may have been overriden by an internal input
-                // with the name id which will KILL IE.
-                // We need to manually iterate at this point because if the case
-                // above is true YUI's ancestor method will also kill IE!
-                var form = select;
-                while (form && form.get('nodeName').toUpperCase() !== 'FORM') {
-                    form = form.ancestor();
-                }
-                return form;
-            })();
-            // Make sure we have the form
-            if (form) {
-                // Create a function to handle our change event
-                var processchange = function(e, paramobject) {
-                    if ((nothing===false || select.get('value') != nothing) && paramobject.lastindex != select.get('selectedIndex')) {
-                        //prevent event bubbling and detach handlers to prevent multiple submissions caused by double clicking
-                        e.halt();
-                        paramobject.eventkeypress.detach();
-                        paramobject.eventblur.detach();
-                        paramobject.eventchangeorblur.detach();
-
-                        this.submit();
-                    }
-                };
-                // Attach the change event to the keypress, blur, and click actions.
-                // We don't use the change event because IE fires it on every arrow up/down
-                // event.... usability
-                var paramobject = new Object();
-                paramobject.lastindex = select.get('selectedIndex');
-                paramobject.eventkeypress = Y.on('key', processchange, select, 'press:13', form, paramobject);
-                paramobject.eventblur = select.on('blur', processchange, form, paramobject);
-                //little hack for chrome that need onChange event instead of onClick - see MDL-23224
-                if (Y.UA.webkit) {
-                    paramobject.eventchangeorblur = select.on('change', processchange, form, paramobject);
-                } else {
-                    paramobject.eventchangeorblur = select.on('click', processchange, form, paramobject);
-                }
-            }
-        }
-    });
-};
-
-/**
- * Attach handler to url_select
- */
-M.util.init_url_select = function(Y, formid, selectid, nothing) {
-    YUI(M.yui.loader).use('node', function(Y) {
-        Y.on('change', function() {
-            if ((nothing == false && Y.Lang.isBoolean(nothing)) || Y.one('#'+selectid).get('value') != nothing) {
-                window.location = M.cfg.wwwroot+Y.one('#'+selectid).get('value');
-            }
-        },
-        '#'+selectid);
-    });
-};
-
 /**
  * Breaks out all links to the top frame - used in frametop page layout.
  */
@@ -647,14 +597,32 @@ M.util.init_block_hider = function(Y, config) {
                     this.set('block', '#'+this.get('id'));
                     var b = this.get('block'),
                         t = b.one('.title'),
-                        a = null;
+                        a = null,
+                        hide,
+                        show;
                     if (t && (a = t.one('.block_action'))) {
-                        var hide = Y.Node.create('<img class="block-hider-hide" tabindex="0" alt="'+config.tooltipVisible+'" title="'+config.tooltipVisible+'" />');
-                        hide.setAttribute('src', this.get('iconVisible')).on('click', this.updateState, this, true);
+                        hide = Y.Node.create('<img />')
+                            .addClass('block-hider-hide')
+                            .setAttrs({
+                                alt:        config.tooltipVisible,
+                                src:        this.get('iconVisible'),
+                                tabindex:   0,
+                                'title':    config.tooltipVisible
+                            });
                         hide.on('keypress', this.updateStateKey, this, true);
-                        var show = Y.Node.create('<img class="block-hider-show" tabindex="0" alt="'+config.tooltipHidden+'" title="'+config.tooltipHidden+'" />');
-                        show.setAttribute('src', this.get('iconHidden')).on('click', this.updateState, this, false);
+                        hide.on('click', this.updateState, this, true);
+
+                        show = Y.Node.create('<img />')
+                            .addClass('block-hider-show')
+                            .setAttrs({
+                                alt:        config.tooltipHidden,
+                                src:        this.get('iconHidden'),
+                                tabindex:   0,
+                                'title':    config.tooltipHidden
+                            });
                         show.on('keypress', this.updateStateKey, this, false);
+                        show.on('click', this.updateState, this, false);
+
                         a.insert(show, 0).insert(hide, 0);
                     }
                 },
@@ -696,6 +664,69 @@ M.util.init_block_hider = function(Y, config) {
     });
 };
 
+/**
+ * @var pending_js - The keys are the list of all pending js actions.
+ * @type Object
+ */
+M.util.pending_js = [];
+M.util.complete_js = [];
+
+/**
+ * Register any long running javascript code with a unique identifier.
+ * Should be followed with a call to js_complete with a matching
+ * idenfitier when the code is complete. May also be called with no arguments
+ * to test if there is any js calls pending. This is relied on by behat so that
+ * it can wait for all pending updates before interacting with a page.
+ * @param String uniqid - optional, if provided,
+ *                        registers this identifier until js_complete is called.
+ * @return boolean - True if there is any pending js.
+ */
+M.util.js_pending = function(uniqid) {
+    if (uniqid !== false) {
+        M.util.pending_js.push(uniqid);
+    }
+
+    return M.util.pending_js.length;
+};
+
+// Start this asap.
+M.util.js_pending('init');
+
+/**
+ * Register listeners for Y.io start/end so we can wait for them in behat.
+ */
+YUI.add('moodle-core-io', function(Y) {
+    Y.on('io:start', function(id) {
+        M.util.js_pending('io:' + id);
+    });
+    Y.on('io:end', function(id) {
+        M.util.js_complete('io:' + id);
+    });
+}, '@VERSION@', {
+    condition: {
+        trigger: 'io-base',
+        when: 'after'
+    }
+});
+
+/**
+ * Unregister any long running javascript code by unique identifier.
+ * This function should form a matching pair with js_pending
+ *
+ * @param String uniqid - required, unregisters this identifier
+ * @return boolean - True if there is any pending js.
+ */
+M.util.js_complete = function(uniqid) {
+    // Use the Y.Array.indexOf instead of the native because some older browsers do not support
+    // the native function. Y.Array polyfills the native function if it does not exist.
+    var index = Y.Array.indexOf(M.util.pending_js, uniqid);
+    if (index >= 0) {
+        M.util.complete_js.push(M.util.pending_js.splice(index, 1));
+    }
+
+    return M.util.pending_js.length;
+};
+
 /**
  * Returns a string registered in advance for usage in JavaScript
  *
@@ -795,6 +826,84 @@ M.util.focus_login_form = function(Y) {
     }
 }
 
+/**
+ * Set focus on login error message
+ */
+M.util.focus_login_error = function(Y) {
+    var errorlog = Y.one('#loginerrormessage');
+
+    if (errorlog) {
+        errorlog.focus();
+    }
+}
+/**
+ * Adds lightbox hidden element that covers the whole node.
+ *
+ * @param {YUI} Y
+ * @param {Node} the node lightbox should be added to
+ * @retun {Node} created lightbox node
+ */
+M.util.add_lightbox = function(Y, node) {
+    var WAITICON = {'pix':"i/loading_small",'component':'moodle'};
+
+    // Check if lightbox is already there
+    if (node.one('.lightbox')) {
+        return node.one('.lightbox');
+    }
+
+    node.setStyle('position', 'relative');
+    var waiticon = Y.Node.create('<img />')
+    .setAttrs({
+        'src' : M.util.image_url(WAITICON.pix, WAITICON.component)
+    })
+    .setStyles({
+        'position' : 'relative',
+        'top' : '50%'
+    });
+
+    var lightbox = Y.Node.create('<div></div>')
+    .setStyles({
+        'opacity' : '.75',
+        'position' : 'absolute',
+        'width' : '100%',
+        'height' : '100%',
+        'top' : 0,
+        'left' : 0,
+        'backgroundColor' : 'white',
+        'textAlign' : 'center'
+    })
+    .setAttribute('class', 'lightbox')
+    .hide();
+
+    lightbox.appendChild(waiticon);
+    node.append(lightbox);
+    return lightbox;
+}
+
+/**
+ * Appends a hidden spinner element to the specified node.
+ *
+ * @param {YUI} Y
+ * @param {Node} the node the spinner should be added to
+ * @return {Node} created spinner node
+ */
+M.util.add_spinner = function(Y, node) {
+    var WAITICON = {'pix':"i/loading_small",'component':'moodle'};
+
+    // Check if spinner is already there
+    if (node.one('.spinner')) {
+        return node.one('.spinner');
+    }
+
+    var spinner = Y.Node.create('<img />')
+        .setAttribute('src', M.util.image_url(WAITICON.pix, WAITICON.component))
+        .addClass('spinner')
+        .addClass('iconsmall')
+        .hide();
+
+    node.append(spinner);
+    return spinner;
+}
 
 //=== old legacy JS code, hopefully to be replaced soon by M.xx.yy and YUI3 code ===
 
@@ -894,8 +1003,14 @@ function findParentNode(el, elName, elClass, elId) {
     specified tag name, class, and id. All conditions must be met,
     but any can be ommitted.
     Doesn't examine children of matches.
+
+    @deprecated since Moodle 2.7 - please do not use this function any more.
+    @todo MDL-43242 This will be deleted in Moodle 2.9.
+    @see Y.all
 */
 function findChildNodes(start, tagName, elementClass, elementID, elementName) {
+    Y.log("findChildNodes() is deprecated. Please use Y.all instead.",
+            "warn", "javascript-static.js");
     var children = new Array();
     for (var i = 0; i < start.childNodes.length; i++) {
         var classfound = false;
@@ -926,34 +1041,46 @@ function findChildNodes(start, tagName, elementClass, elementID, elementName) {
 }
 
 function unmaskPassword(id) {
-  var pw = document.getElementById(id);
-  var chb = document.getElementById(id+'unmask');
+    var pw = document.getElementById(id);
+    var chb = document.getElementById(id+'unmask');
 
-  try {
-    // first try IE way - it can not set name attribute later
-    if (chb.checked) {
-      var newpw = document.createElement('<input type="text" autocomplete="off" name="'+pw.name+'">');
-    } else {
-      var newpw = document.createElement('<input type="password" autocomplete="off" name="'+pw.name+'">');
+    // MDL-30438 - The capability to changing the value of input type is not supported by IE8 or lower.
+    // Replacing existing child with a new one, removed all yui properties for the node.  Therefore, this
+    // functionality won't work in IE8 or lower.
+    // This is a temporary fixed to allow other browsers to function properly.
+    if (Y.UA.ie == 0 || Y.UA.ie >= 9) {
+        if (chb.checked) {
+            pw.type = "text";
+        } else {
+            pw.type = "password";
+        }
+    } else {  //IE Browser version 8 or lower
+        try {
+            // first try IE way - it can not set name attribute later
+            if (chb.checked) {
+              var newpw = document.createElement('<input type="text" autocomplete="off" name="'+pw.name+'">');
+            } else {
+              var newpw = document.createElement('<input type="password" autocomplete="off" name="'+pw.name+'">');
+            }
+            newpw.attributes['class'].nodeValue = pw.attributes['class'].nodeValue;
+        } catch (e) {
+            var newpw = document.createElement('input');
+            newpw.setAttribute('autocomplete', 'off');
+            newpw.setAttribute('name', pw.name);
+            if (chb.checked) {
+              newpw.setAttribute('type', 'text');
+            } else {
+              newpw.setAttribute('type', 'password');
+            }
+            newpw.setAttribute('class', pw.getAttribute('class'));
+        }
+        newpw.id = pw.id;
+        newpw.size = pw.size;
+        newpw.onblur = pw.onblur;
+        newpw.onchange = pw.onchange;
+        newpw.value = pw.value;
+        pw.parentNode.replaceChild(newpw, pw);
     }
-    newpw.attributes['class'].nodeValue = pw.attributes['class'].nodeValue;
-  } catch (e) {
-    var newpw = document.createElement('input');
-    newpw.setAttribute('autocomplete', 'off');
-    newpw.setAttribute('name', pw.name);
-    if (chb.checked) {
-      newpw.setAttribute('type', 'text');
-    } else {
-      newpw.setAttribute('type', 'password');
-    }
-    newpw.setAttribute('class', pw.getAttribute('class'));
-  }
-  newpw.id = pw.id;
-  newpw.size = pw.size;
-  newpw.onblur = pw.onblur;
-  newpw.onchange = pw.onchange;
-  newpw.value = pw.value;
-  pw.parentNode.replaceChild(newpw, pw);
 }
 
 function filterByParent(elCollection, parentFinder) {
@@ -1033,8 +1160,11 @@ function insertAtCursor(myField, myValue) {
         Call instead of setting window.onload directly or setting body onload=.
         Adds your function to a chain of functions rather than overwriting anything
         that exists.
+        @deprecated Since Moodle 2.7. This will be removed in Moodle 2.9.
 */
 function addonload(fn) {
+    Y.log('addonload has been deprecated since Moodle 2.7 and will be removed in Moodle 2.9',
+            'warn', 'javascript-static.js');
     var oldhandler=window.onload;
     window.onload=function() {
         if(oldhandler) oldhandler();
@@ -1055,8 +1185,11 @@ function addonload(fn) {
  *                    document, use `document`.
  * @param {String} strTagName filter by tag names
  * @param {String} name same as HTML5 spec
+ * @deprecated Since Moodle 2.7. This will be removed in Moodle 2.9.
  */
 function getElementsByClassName(oElm, strTagName, name) {
+    Y.log('getElementsByClassName has been deprecated since Moodle 2.7 and will be removed in Moodle 2.9',
+            'warn', 'javascript-static.js');
     // for backwards compatibility
     if(typeof name == "object") {
         var names = new Array();
@@ -1099,6 +1232,55 @@ function getElementsByClassName(oElm, strTagName, name) {
     return (arrReturnElements)
 }
 
+/**
+ * Increment a file name.
+ *
+ * @param string file name.
+ * @param boolean ignoreextension do not extract the extension prior to appending the
+ *                                suffix. Useful when incrementing folder names.
+ * @return string the incremented file name.
+ */
+function increment_filename(filename, ignoreextension) {
+    var extension = '';
+    var basename = filename;
+
+    // Split the file name into the basename + extension.
+    if (!ignoreextension) {
+        var dotpos = filename.lastIndexOf('.');
+        if (dotpos !== -1) {
+            basename = filename.substr(0, dotpos);
+            extension = filename.substr(dotpos, filename.length);
+        }
+    }
+
+    // Look to see if the name already has (NN) at the end of it.
+    var number = 0;
+    var hasnumber = basename.match(/^(.*) \((\d+)\)$/);
+    if (hasnumber !== null) {
+        // Note the current number & remove it from the basename.
+        number = parseInt(hasnumber[2], 10);
+        basename = hasnumber[1];
+    }
+
+    number++;
+    var newname = basename + ' (' + number + ')' + extension;
+    return newname;
+}
+
+/**
+ * Return whether we are in right to left mode or not.
+ *
+ * @return boolean
+ */
+function right_to_left() {
+    var body = Y.one('body');
+    var rtl = false;
+    if (body && body.hasClass('dir-rtl')) {
+        rtl = true;
+    }
+    return rtl;
+}
+
 function openpopup(event, args) {
 
     if (event) {
@@ -1115,10 +1297,10 @@ function openpopup(event, args) {
         args.name = '_blank';
     } else if (args.name.match(nameregex)) {
         // Cleans window name because IE does not support funky ones.
-        args.name = args.name.replace(nameregex, '_');
         if (M.cfg.developerdebug) {
-            alert('DEVELOPER NOTICE: Invalid \'name\' passed to openpopup()');
+            alert('DEVELOPER NOTICE: Invalid \'name\' passed to openpopup(): ' + args.name);
         }
+        args.name = args.name.replace(nameregex, '_');
     }
 
     var fullurl = args.url;
@@ -1179,19 +1361,24 @@ function close_window(e) {
 
 /**
  * Used in a couple of modules to hide navigation areas when using AJAX
+ * @deprecated since Moodle 2.7. This function will be removed in Moodle 2.9.
  */
-
 function show_item(itemid) {
-    var item = document.getElementById(itemid);
+    Y.log('show_item has been deprecated since Moodle 2.7 and will be removed in Moodle 2.9',
+            'warn', 'javascript-static.js');
+    var item = Y.one('#' + itemid);
     if (item) {
-        item.style.display = "";
+        item.show();
     }
 }
 
+// Deprecated since Moodle 2.7. This function will be removed in Moodle 2.9.
 function destroy_item(itemid) {
-    var item = document.getElementById(itemid);
+    Y.log('destroy_item has been deprecated since Moodle 2.7 and will be removed in Moodle 2.9',
+            'warn', 'javascript-static.js');
+    var item = Y.one('#' + itemid);
     if (item) {
-        item.parentNode.removeChild(item);
+        item.remove(true);
     }
 }
 /**
@@ -1284,134 +1471,52 @@ function update_progress_bar (id, width, pt, msg, es){
 
 /**
  * Used in a couple of modules to hide navigation areas when using AJAX
+ * @deprecated since Moodle 2.7. This function will be removed in Moodle 2.9.
  */
 function hide_item(itemid) {
-    // use class='hiddenifjs' instead
-    var item = document.getElementById(itemid);
+    Y.log('hide_item has been deprecated since Moodle 2.7 and will be removed in Moodle 2.9',
+            'warn', 'javascript-static.js');
+    var item = Y.one('#' + itemid);
     if (item) {
-        item.style.display = "none";
+        item.hide();
     }
 }
 
-M.util.help_icon = {
-    Y : null,
-    instance : null,
-    add : function(Y, properties) {
-        this.Y = Y;
-        properties.node = Y.one('#'+properties.id);
-        if (properties.node) {
-            properties.node.on('click', this.display, this, properties);
-        }
+M.util.help_popups = {
+    setup : function(Y) {
+        Y.one('body').delegate('click', this.open_popup, 'a.helplinkpopup', this);
     },
-    display : function(event, args) {
-        event.preventDefault();
-        if (M.util.help_icon.instance === null) {
-            var Y = M.util.help_icon.Y;
-            Y.use('overlay', 'io-base', 'event-mouseenter', 'node', 'event-key', 'escape', function(Y) {
-                var help_content_overlay = {
-                    helplink : null,
-                    overlay : null,
-                    init : function() {
+    open_popup : function(e) {
+        // Prevent the default page action
+        e.preventDefault();
 
-                        var strclose = Y.Escape.html(M.str.form.close);
-                        var footerbtn = Y.Node.create('<button class="closebtn">'+strclose+'</button>');
-                        // Create an overlay from markup
-                        this.overlay = new Y.Overlay({
-                            footerContent: footerbtn,
-                            bodyContent: '',
-                            id: 'helppopupbox',
-                            width:'400px',
-                            visible : false,
-                            constrain : true
-                        });
-                        this.overlay.render(Y.one(document.body));
+        // Grab the anchor that was clicked
+        var anchor = e.target.ancestor('a', true);
+        var args = {
+            'name'          : 'popup',
+            'url'           : anchor.getAttribute('href'),
+            'options'       : ''
+        };
+        var options = [
+            'height=600',
+            'width=800',
+            'top=0',
+            'left=0',
+            'menubar=0',
+            'location=0',
+            'scrollbars',
+            'resizable',
+            'toolbar',
+            'status',
+            'directories=0',
+            'fullscreen=0',
+            'dependent'
+        ]
+        args.options = options.join(',');
 
-                        footerbtn.on('click', this.overlay.hide, this.overlay);
-
-                        var boundingBox = this.overlay.get("boundingBox");
-
-                        //  Hide the menu if the user clicks outside of its content
-                        boundingBox.get("ownerDocument").on("mousedown", function (event) {
-                            var oTarget = event.target;
-                            var menuButton = Y.one("#"+args.id);
-
-                            if (!oTarget.compareTo(menuButton) &&
-                                !menuButton.contains(oTarget) &&
-                                !oTarget.compareTo(boundingBox) &&
-                                !boundingBox.contains(oTarget)) {
-                                this.overlay.hide();
-                            }
-                        }, this);
-                    },
-
-                    close : function(e) {
-                        e.preventDefault();
-                        this.helplink.focus();
-                        this.overlay.hide();
-                    },
-
-                    display : function(event, args) {
-                        if (Y.one('html').get('dir') == 'rtl') {
-                            var overlayPosition = [Y.WidgetPositionAlign.TR, Y.WidgetPositionAlign.LC];
-                        } else {
-                            var overlayPosition = [Y.WidgetPositionAlign.TL, Y.WidgetPositionAlign.RC];
-                        }
-
-                        this.helplink = args.node;
-
-                        this.overlay.set('bodyContent', Y.Node.create('<img src="'+M.cfg.loadingicon+'" class="spinner" />'));
-                        this.overlay.set("align", {node:args.node, points: overlayPosition});
-
-                        var fullurl = args.url;
-                        if (!args.url.match(/https?:\/\//)) {
-                            fullurl = M.cfg.wwwroot + args.url;
-                        }
-
-                        var ajaxurl = fullurl + '&ajax=1';
-
-                        var cfg = {
-                            method: 'get',
-                            context : this,
-                            on: {
-                                success: function(id, o, node) {
-                                    this.display_callback(o.responseText);
-                                },
-                                failure: function(id, o, node) {
-                                    var debuginfo = o.statusText;
-                                    if (M.cfg.developerdebug) {
-                                        o.statusText += ' (' + ajaxurl + ')';
-                                    }
-                                    this.display_callback('bodyContent',debuginfo);
-                                }
-                            }
-                        };
-
-                        Y.io(ajaxurl, cfg);
-                        this.overlay.show();
-                    },
-
-                    display_callback : function(content) {
-                        content = '<div role="alert">' + content + '</div>';
-                        this.overlay.set('bodyContent', content);
-                    },
-
-                    hideContent : function() {
-                        help = this;
-                        help.overlay.hide();
-                    }
-                };
-                help_content_overlay.init();
-                M.util.help_icon.instance = help_content_overlay;
-                M.util.help_icon.instance.display(event, args);
-            });
-        } else {
-            M.util.help_icon.instance.display(event, args);
-        }
-    },
-    init : function(Y) {
-        this.Y = Y;
+        openpopup(e, args);
     }
-};
+}
 
 /**
  * Custom menu namespace
@@ -1701,9 +1806,9 @@ M.util.load_flowplayer = function() {
             for(var i=0; i<M.util.video_players.length; i++) {
                 var video = M.util.video_players[i];
                 if (video.width > 0 && video.height > 0) {
-                    var src = {src: M.cfg.wwwroot + '/lib/flowplayer/flowplayer-3.2.7.swf', width: video.width, height: video.height};
+                    var src = {src: M.cfg.wwwroot + '/lib/flowplayer/flowplayer-3.2.18.swf', width: video.width, height: video.height};
                 } else {
-                    var src = M.cfg.wwwroot + '/lib/flowplayer/flowplayer-3.2.7.swf';
+                    var src = M.cfg.wwwroot + '/lib/flowplayer/flowplayer-3.2.18.swf';
                 }
                 flowplayer(video.id, src, {
                     plugins: {controls: controls},
@@ -1731,7 +1836,7 @@ M.util.load_flowplayer = function() {
                                 object.width = width;
                                 object.height = height;
                             }
-		                }
+                        }
                     }
                 });
             }
@@ -1803,17 +1908,17 @@ M.util.load_flowplayer = function() {
                     controls.height = 25;
                     controls.time = true;
                 }
-                flowplayer(audio.id, M.cfg.wwwroot + '/lib/flowplayer/flowplayer-3.2.7.swf', {
-                    plugins: {controls: controls, audio: {url: M.cfg.wwwroot + '/lib/flowplayer/flowplayer.audio-3.2.2.swf'}},
+                flowplayer(audio.id, M.cfg.wwwroot + '/lib/flowplayer/flowplayer-3.2.18.swf', {
+                    plugins: {controls: controls, audio: {url: M.cfg.wwwroot + '/lib/flowplayer/flowplayer.audio-3.2.11.swf'}},
                     clip: {url: audio.fileurl, provider: "audio", autoPlay: false}
                 });
             }
         }
 
-        if (M.cfg.jsrev == -10) {
-            var jsurl = M.cfg.wwwroot + '/lib/flowplayer/flowplayer-3.2.6.js';
+        if (M.cfg.jsrev == -1) {
+            var jsurl = M.cfg.wwwroot + '/lib/flowplayer/flowplayer-3.2.13.js';
         } else {
-            var jsurl = M.cfg.wwwroot + '/lib/javascript.php?file=/lib/flowplayer/flowplayer-3.2.6.js&rev=' + M.cfg.jsrev;
+            var jsurl = M.cfg.wwwroot + '/lib/javascript.php?jsfile=/lib/flowplayer/flowplayer-3.2.13.min.js&rev=' + M.cfg.jsrev;
         }
         var fileref = document.createElement('script');
         fileref.setAttribute('type','text/javascript');
@@ -1822,4 +1927,4 @@ M.util.load_flowplayer = function() {
         fileref.onreadystatechange = embed_function;
         document.getElementsByTagName('head')[0].appendChild(fileref);
     }
-}
+};
