@@ -1263,9 +1263,7 @@ class mod_assign_external extends external_api {
      * @since Moodle 2.6
      */
     public static function lock_submissions_returns() {
-        return new external_multiple_structure(
-           new external_warnings()
-        );
+        return new external_warnings();
     }
 
     /**
@@ -1327,9 +1325,7 @@ class mod_assign_external extends external_api {
      * @since Moodle 2.6
      */
     public static function revert_submissions_to_draft_returns() {
-        return new external_multiple_structure(
-           new external_warnings()
-        );
+        return new external_warnings();
     }
 
     /**
@@ -1391,9 +1387,7 @@ class mod_assign_external extends external_api {
      * @since Moodle 2.6
      */
     public static function unlock_submissions_returns() {
-        return new external_multiple_structure(
-           new external_warnings()
-        );
+        return new external_warnings();
     }
 
     /**
@@ -1453,9 +1447,7 @@ class mod_assign_external extends external_api {
      * @since Moodle 2.6
      */
     public static function submit_for_grading_returns() {
-        return new external_multiple_structure(
-           new external_warnings()
-        );
+        return new external_warnings();
     }
 
     /**
@@ -1533,9 +1525,7 @@ class mod_assign_external extends external_api {
      * @since Moodle 2.6
      */
     public static function save_user_extensions_returns() {
-        return new external_multiple_structure(
-           new external_warnings()
-        );
+        return new external_warnings();
     }
 
     /**
@@ -1589,9 +1579,7 @@ class mod_assign_external extends external_api {
      * @since Moodle 2.6
      */
     public static function reveal_identities_returns() {
-        return new external_multiple_structure(
-           new external_warnings()
-        );
+        return new external_warnings();
     }
 
     /**
@@ -1667,9 +1655,7 @@ class mod_assign_external extends external_api {
      * @since Moodle 2.6
      */
     public static function save_submission_returns() {
-        return new external_multiple_structure(
-           new external_warnings()
-        );
+        return new external_warnings();
     }
 
     /**
@@ -2017,8 +2003,6 @@ class mod_assign_external extends external_api {
      * @since Moodle 2.6
      */
     public static function copy_previous_attempt_returns() {
-        return new external_multiple_structure(
-           new external_warnings()
-        );
+        return new external_warnings();
     }
 }
