@@ -17,8 +17,7 @@
 /**
  * Defines the quiz module ettings form.
  *
- * @package    mod
- * @subpackage quiz
+ * @package    mod_quiz
  * @copyright  2006 Jamie Pratt
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
@@ -37,18 +36,24 @@ require_once($CFG->dirroot . '/mod/quiz/locallib.php');
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 class mod_quiz_mod_form extends moodleform_mod {
+    /** @var array options to be used with date_time_selector fields in the quiz. */
+    public static $datefieldoptions = array('optional' => true, 'step' => 1);
+
     protected $_feedbacks;
     protected static $reviewfields = array(); // Initialised in the constructor.
 
+    /** @var int the max number of attempts allowed in any user or group override on this quiz. */
+    protected $maxattemptsanyoverride = null;
+
     public function __construct($current, $section, $cm, $course) {
         self::$reviewfields = array(
-            'attempt' => get_string('theattempt', 'quiz'),
-            'correctness' => get_string('whethercorrect', 'question'),
-            'marks' => get_string('marks', 'question'),
-            'specificfeedback' => get_string('specificfeedback', 'question'),
-            'generalfeedback' => get_string('generalfeedback', 'question'),
-            'rightanswer' => get_string('rightanswer', 'question'),
-            'overallfeedback' => get_string('overallfeedback', 'quiz'),
+            'attempt'          => array('theattempt', 'quiz'),
+            'correctness'      => array('whethercorrect', 'question'),
+            'marks'            => array('marks', 'quiz'),
+            'specificfeedback' => array('specificfeedback', 'question'),
+            'generalfeedback'  => array('generalfeedback', 'question'),
+            'rightanswer'      => array('rightanswer', 'question'),
+            'overallfeedback'  => array('reviewoverallfeedback', 'quiz'),
         );
         parent::__construct($current, $section, $cm, $course);
     }
@@ -58,7 +63,7 @@ class mod_quiz_mod_form extends moodleform_mod {
         $quizconfig = get_config('quiz');
         $mform = $this->_form;
 
-        //-------------------------------------------------------------------------------
+        // -------------------------------------------------------------------------------
         $mform->addElement('header', 'general', get_string('general', 'form'));
 
         // Name.
@@ -69,17 +74,21 @@ class mod_quiz_mod_form extends moodleform_mod {
             $mform->setType('name', PARAM_CLEANHTML);
         }
         $mform->addRule('name', null, 'required', null, 'client');
+        $mform->addRule('name', get_string('maximumchars', '', 255), 'maxlength', 255, 'client');
 
         // Introduction.
         $this->add_intro_editor(false, get_string('introduction', 'quiz'));
 
+        // -------------------------------------------------------------------------------
+        $mform->addElement('header', 'timing', get_string('timing', 'quiz'));
+
         // Open and close dates.
         $mform->addElement('date_time_selector', 'timeopen', get_string('quizopen', 'quiz'),
-                array('optional' => true, 'step' => 1));
+                self::$datefieldoptions);
         $mform->addHelpButton('timeopen', 'quizopenclose', 'quiz');
 
         $mform->addElement('date_time_selector', 'timeclose', get_string('quizclose', 'quiz'),
-                array('optional' => true, 'step' => 1));
+                self::$datefieldoptions);
 
         // Time limit.
         $mform->addElement('duration', 'timelimit', get_string('timelimit', 'quiz'),
@@ -88,6 +97,37 @@ class mod_quiz_mod_form extends moodleform_mod {
         $mform->setAdvanced('timelimit', $quizconfig->timelimit_adv);
         $mform->setDefault('timelimit', $quizconfig->timelimit);
 
+        // What to do with overdue attempts.
+        $mform->addElement('select', 'overduehandling', get_string('overduehandling', 'quiz'),
+                quiz_get_overdue_handling_options());
+        $mform->addHelpButton('overduehandling', 'overduehandling', 'quiz');
+        $mform->setAdvanced('overduehandling', $quizconfig->overduehandling_adv);
+        $mform->setDefault('overduehandling', $quizconfig->overduehandling);
+        // TODO Formslib does OR logic on disableif, and we need AND logic here.
+        // $mform->disabledIf('overduehandling', 'timelimit', 'eq', 0);
+        // $mform->disabledIf('overduehandling', 'timeclose', 'eq', 0);
+
+        // Grace period time.
+        $mform->addElement('duration', 'graceperiod', get_string('graceperiod', 'quiz'),
+                array('optional' => true));
+        $mform->addHelpButton('graceperiod', 'graceperiod', 'quiz');
+        $mform->setAdvanced('graceperiod', $quizconfig->graceperiod_adv);
+        $mform->setDefault('graceperiod', $quizconfig->graceperiod);
+        $mform->disabledIf('graceperiod', 'overduehandling', 'neq', 'graceperiod');
+
+        // -------------------------------------------------------------------------------
+        // Grade settings.
+        $this->standard_grading_coursemodule_elements();
+
+        $mform->removeElement('grade');
+        if (property_exists($this->current, 'grade')) {
+            $currentgrade = $this->current->grade;
+        } else {
+            $currentgrade = $quizconfig->maximumgrade;
+        }
+        $mform->addElement('hidden', 'grade', $currentgrade);
+        $mform->setType('grade', PARAM_FLOAT);
+
         // Number of attempts.
         $attemptoptions = array('0' => get_string('unlimited'));
         for ($i = 1; $i <= QUIZ_MAX_ATTEMPT_OPTION; $i++) {
@@ -104,17 +144,11 @@ class mod_quiz_mod_form extends moodleform_mod {
         $mform->addHelpButton('grademethod', 'grademethod', 'quiz');
         $mform->setAdvanced('grademethod', $quizconfig->grademethod_adv);
         $mform->setDefault('grademethod', $quizconfig->grademethod);
-        $mform->disabledIf('grademethod', 'attempts', 'eq', 1);
+        if ($this->get_max_attempts_for_any_override() < 2) {
+            $mform->disabledIf('grademethod', 'attempts', 'eq', 1);
+        }
 
-        //-------------------------------------------------------------------------------
-        // Grade settings
-        $this->standard_grading_coursemodule_elements();
-
-        $mform->removeElement('grade');
-        $mform->addElement('hidden', 'grade', $quizconfig->maximumgrade);
-        $mform->setType('grade', PARAM_NUMBER);
-
-        //-------------------------------------------------------------------------------
+        // -------------------------------------------------------------------------------
         $mform->addElement('header', 'layouthdr', get_string('layout', 'quiz'));
 
         // Shuffle questions.
@@ -144,9 +178,16 @@ class mod_quiz_mod_form extends moodleform_mod {
             $pagegroup[] = $mform->createElement('checkbox', 'repaginatenow', '',
                     get_string('repaginatenow', 'quiz'), array('id' => 'id_repaginatenow'));
             $mform->disabledIf('repaginatenow', 'shufflequestions', 'eq', 1);
-            $PAGE->requires->yui2_lib('event');
-            $PAGE->requires->js('/mod/quiz/edit.js');
-            $PAGE->requires->js_init_call('quiz_settings_init');
+
+            $PAGE->requires->js('/question/qengine.js');
+            $module = array(
+                'name'      => 'mod_quiz_edit',
+                'fullpath'  => '/mod/quiz/edit.js',
+                'requires'  => array('yui2-dom', 'yui2-event', 'yui2-container'),
+                'strings'   => array(),
+                'async'     => false,
+            );
+            $PAGE->requires->js_init_call('quiz_settings_init', null, false, $module);
         }
 
         $mform->addGroup($pagegroup, 'questionsperpagegrp',
@@ -154,7 +195,14 @@ class mod_quiz_mod_form extends moodleform_mod {
         $mform->addHelpButton('questionsperpagegrp', 'newpage', 'quiz');
         $mform->setAdvanced('questionsperpagegrp', $quizconfig->questionsperpage_adv);
 
-        //-------------------------------------------------------------------------------
+        // Navigation method.
+        $mform->addElement('select', 'navmethod', get_string('navmethod', 'quiz'),
+                quiz_get_navigation_options());
+        $mform->addHelpButton('navmethod', 'navmethod', 'quiz');
+        $mform->setAdvanced('navmethod', $quizconfig->navmethod_adv);
+        $mform->setDefault('navmethod', $quizconfig->navmethod);
+
+        // -------------------------------------------------------------------------------
         $mform->addElement('header', 'interactionhdr', get_string('questionbehaviour', 'quiz'));
 
         // Shuffle within questions.
@@ -181,16 +229,18 @@ class mod_quiz_mod_form extends moodleform_mod {
         $mform->addHelpButton('attemptonlast', 'eachattemptbuildsonthelast', 'quiz');
         $mform->setAdvanced('attemptonlast', $quizconfig->attemptonlast_adv);
         $mform->setDefault('attemptonlast', $quizconfig->attemptonlast);
-        $mform->disabledIf('attemptonlast', 'attempts', 'eq', 1);
+        if ($this->get_max_attempts_for_any_override() < 2) {
+            $mform->disabledIf('attemptonlast', 'attempts', 'eq', 1);
+        }
 
-        //-------------------------------------------------------------------------------
+        // -------------------------------------------------------------------------------
         $mform->addElement('header', 'reviewoptionshdr',
                 get_string('reviewoptionsheading', 'quiz'));
         $mform->addHelpButton('reviewoptionshdr', 'reviewoptionsheading', 'quiz');
 
         // Review options.
         $this->add_review_options_group($mform, $quizconfig, 'during',
-                mod_quiz_display_options::DURING);
+                mod_quiz_display_options::DURING, true);
         $this->add_review_options_group($mform, $quizconfig, 'immediately',
                 mod_quiz_display_options::IMMEDIATELY_AFTER);
         $this->add_review_options_group($mform, $quizconfig, 'open',
@@ -210,12 +260,12 @@ class mod_quiz_mod_form extends moodleform_mod {
         $mform->disabledIf('overallfeedbackduring', 'preferredbehaviour',
                 'neq', 'wontmatch');
 
-        //-------------------------------------------------------------------------------
-        $mform->addElement('header', 'display', get_string('display', 'form'));
+        // -------------------------------------------------------------------------------
+        $mform->addElement('header', 'display', get_string('appearance'));
 
         // Show user picture.
-        $mform->addElement('selectyesno', 'showuserpicture',
-                get_string('showuserpicture', 'quiz'));
+        $mform->addElement('select', 'showuserpicture', get_string('showuserpicture', 'quiz'),
+                quiz_get_user_image_options());
         $mform->addHelpButton('showuserpicture', 'showuserpicture', 'quiz');
         $mform->setAdvanced('showuserpicture', $quizconfig->showuserpicture_adv);
         $mform->setDefault('showuserpicture', $quizconfig->showuserpicture);
@@ -242,13 +292,13 @@ class mod_quiz_mod_form extends moodleform_mod {
         $mform->setAdvanced('questiondecimalpoints', $quizconfig->questiondecimalpoints_adv);
         $mform->setDefault('questiondecimalpoints', $quizconfig->questiondecimalpoints);
 
-        // Show blocks during quiz attempt
+        // Show blocks during quiz attempt.
         $mform->addElement('selectyesno', 'showblocks', get_string('showblocks', 'quiz'));
         $mform->addHelpButton('showblocks', 'showblocks', 'quiz');
         $mform->setAdvanced('showblocks', $quizconfig->showblocks_adv);
         $mform->setDefault('showblocks', $quizconfig->showblocks);
 
-        //-------------------------------------------------------------------------------
+        // -------------------------------------------------------------------------------
         $mform->addElement('header', 'security', get_string('extraattemptrestrictions', 'quiz'));
 
         // Require password to begin quiz attempt.
@@ -271,17 +321,21 @@ class mod_quiz_mod_form extends moodleform_mod {
         $mform->addHelpButton('delay1', 'delay1st2nd', 'quiz');
         $mform->setAdvanced('delay1', $quizconfig->delay1_adv);
         $mform->setDefault('delay1', $quizconfig->delay1);
-        $mform->disabledIf('delay1', 'attempts', 'eq', 1);
+        if ($this->get_max_attempts_for_any_override() < 2) {
+            $mform->disabledIf('delay1', 'attempts', 'eq', 1);
+        }
 
         $mform->addElement('duration', 'delay2', get_string('delaylater', 'quiz'),
                 array('optional' => true));
         $mform->addHelpButton('delay2', 'delaylater', 'quiz');
         $mform->setAdvanced('delay2', $quizconfig->delay2_adv);
         $mform->setDefault('delay2', $quizconfig->delay2);
-        $mform->disabledIf('delay2', 'attempts', 'eq', 1);
-        $mform->disabledIf('delay2', 'attempts', 'eq', 2);
+        if ($this->get_max_attempts_for_any_override() < 3) {
+            $mform->disabledIf('delay2', 'attempts', 'eq', 1);
+            $mform->disabledIf('delay2', 'attempts', 'eq', 2);
+        }
 
-        // 'Secure' window.
+        // Browser security choices.
         $mform->addElement('select', 'browsersecurity', get_string('browsersecurity', 'quiz'),
                 quiz_access_manager::get_browser_security_choices());
         $mform->addHelpButton('browsersecurity', 'browsersecurity', 'quiz');
@@ -291,7 +345,7 @@ class mod_quiz_mod_form extends moodleform_mod {
         // Any other rule plugins.
         quiz_access_manager::add_settings_form_fields($this, $mform);
 
-        //-------------------------------------------------------------------------------
+        // -------------------------------------------------------------------------------
         $mform->addElement('header', 'overallfeedbackhdr', get_string('overallfeedback', 'quiz'));
         $mform->addHelpButton('overallfeedbackhdr', 'overallfeedback', 'quiz');
 
@@ -310,10 +364,10 @@ class mod_quiz_mod_form extends moodleform_mod {
 
         $repeatarray = array();
         $repeatedoptions = array();
-        $repeatarray[] = MoodleQuickForm::createElement('editor', 'feedbacktext',
-                get_string('feedback', 'quiz'), null, array('maxfiles' => EDITOR_UNLIMITED_FILES,
+        $repeatarray[] = $mform->createElement('editor', 'feedbacktext',
+                get_string('feedback', 'quiz'), array('rows' => 3), array('maxfiles' => EDITOR_UNLIMITED_FILES,
                         'noclean' => true, 'context' => $this->context));
-        $repeatarray[] = MoodleQuickForm::createElement('text', 'feedbackboundaries',
+        $repeatarray[] = $mform->createElement('text', 'feedbackboundaries',
                 get_string('gradeboundary', 'quiz'), array('size' => 10));
         $repeatedoptions['feedbacktext']['type'] = PARAM_RAW;
         $repeatedoptions['feedbackboundaries']['type'] = PARAM_RAW;
@@ -330,13 +384,13 @@ class mod_quiz_mod_form extends moodleform_mod {
                 $repeatedoptions, 'boundary_repeats', 'boundary_add_fields', 3,
                 get_string('addmoreoverallfeedbacks', 'quiz'), true);
 
-        // Put some extra elements in before the button
-        $mform->insertElementBefore(MoodleQuickForm::createElement('editor',
-                "feedbacktext[$nextel]", get_string('feedback', 'quiz'), null,
+        // Put some extra elements in before the button.
+        $mform->insertElementBefore($mform->createElement('editor',
+                "feedbacktext[$nextel]", get_string('feedback', 'quiz'), array('rows' => 3),
                 array('maxfiles' => EDITOR_UNLIMITED_FILES, 'noclean' => true,
                       'context' => $this->context)),
                 'boundary_add_fields');
-        $mform->insertElementBefore(MoodleQuickForm::createElement('static',
+        $mform->insertElementBefore($mform->createElement('static',
                 'gradeboundarystatic2', get_string('gradeboundary', 'quiz'), '0%'),
                 'boundary_add_fields');
 
@@ -347,17 +401,33 @@ class mod_quiz_mod_form extends moodleform_mod {
             $mform->disabledIf('feedbacktext[' . ($i + 1) . ']', 'grade', 'eq', 0);
         }
 
-        //-------------------------------------------------------------------------------
+        // -------------------------------------------------------------------------------
         $this->standard_coursemodule_elements();
 
-        //-------------------------------------------------------------------------------
-        // buttons
+        // Check and act on whether setting outcomes is considered an advanced setting.
+        $mform->setAdvanced('modoutcomes', !empty($quizconfig->outcomes_adv));
+
+        // The standard_coursemodule_elements method sets this to 100, but the
+        // quiz has its own setting, so use that.
+        $mform->setDefault('grade', $quizconfig->maximumgrade);
+
+        // -------------------------------------------------------------------------------
         $this->add_action_buttons();
     }
 
-    protected function add_review_options_group($mform, $quizconfig, $whenname, $when) {
+    protected function add_review_options_group($mform, $quizconfig, $whenname,
+            $when, $withhelp = false) {
+        global $OUTPUT;
+
         $group = array();
-        foreach (self::$reviewfields as $field => $label) {
+        foreach (self::$reviewfields as $field => $string) {
+            list($identifier, $component) = $string;
+
+            $label = get_string($identifier, $component);
+            if ($withhelp) {
+                $label .= ' ' . $OUTPUT->help_icon($identifier, $component);
+            }
+
             $group[] = $mform->createElement('checkbox', $field . $whenname, '', $label);
         }
         $mform->addGroup($group, $whenname . 'optionsgrp',
@@ -372,10 +442,12 @@ class mod_quiz_mod_form extends moodleform_mod {
             }
         }
 
-        $mform->disabledIf('correctness' . $whenname, 'attempt' . $whenname);
-        $mform->disabledIf('specificfeedback' . $whenname, 'attempt' . $whenname);
-        $mform->disabledIf('generalfeedback' . $whenname, 'attempt' . $whenname);
-        $mform->disabledIf('rightanswer' . $whenname, 'attempt' . $whenname);
+        if ($whenname != 'during') {
+            $mform->disabledIf('correctness' . $whenname, 'attempt' . $whenname);
+            $mform->disabledIf('specificfeedback' . $whenname, 'attempt' . $whenname);
+            $mform->disabledIf('generalfeedback' . $whenname, 'attempt' . $whenname);
+            $mform->disabledIf('rightanswer' . $whenname, 'attempt' . $whenname);
+        }
     }
 
     protected function preprocessing_review_settings(&$toform, $whenname, $when) {
@@ -398,13 +470,13 @@ class mod_quiz_mod_form extends moodleform_mod {
             foreach ($this->_feedbacks as $feedback) {
                 $draftid = file_get_submitted_draft_itemid('feedbacktext['.$key.']');
                 $toform['feedbacktext['.$key.']']['text'] = file_prepare_draft_area(
-                    $draftid,               // draftid
-                    $this->context->id,     // context
-                    'mod_quiz',             // component
-                    'feedback',             // filarea
-                    !empty($feedback->id) ? (int) $feedback->id : null, // itemid
+                    $draftid,               // Draftid.
+                    $this->context->id,     // Context.
+                    'mod_quiz',             // Component.
+                    'feedback',             // Filarea.
+                    !empty($feedback->id) ? (int) $feedback->id : null, // Itemid.
                     null,
-                    $feedback->feedbacktext // text
+                    $feedback->feedbacktext // Text.
                 );
                 $toform['feedbacktext['.$key.']']['format'] = $feedback->feedbacktextformat;
                 $toform['feedbacktext['.$key.']']['itemid'] = $draftid;
@@ -420,7 +492,7 @@ class mod_quiz_mod_form extends moodleform_mod {
 
                 if ($feedback->mingrade > 0) {
                     $toform['feedbackboundaries['.$key.']'] =
-                            (100.0 * $feedback->mingrade / $toform['grade']) . '%';
+                            round(100.0 * $feedback->mingrade / $toform['grade'], 6) . '%';
                 }
                 $key++;
             }
@@ -466,15 +538,28 @@ class mod_quiz_mod_form extends moodleform_mod {
             $errors['timeclose'] = get_string('closebeforeopen', 'quiz');
         }
 
+        // Check that the grace period is not too short.
+        if ($data['overduehandling'] == 'graceperiod') {
+            $graceperiodmin = get_config('quiz', 'graceperiodmin');
+            if ($data['graceperiod'] <= $graceperiodmin) {
+                $errors['graceperiod'] = get_string('graceperiodtoosmall', 'quiz', format_time($graceperiodmin));
+            }
+        }
+
         // Check the boundary value is a number or a percentage, and in range.
         $i = 0;
         while (!empty($data['feedbackboundaries'][$i] )) {
             $boundary = trim($data['feedbackboundaries'][$i]);
-            if (strlen($boundary) > 0 && $boundary[strlen($boundary) - 1] == '%') {
-                $boundary = trim(substr($boundary, 0, -1));
-                if (is_numeric($boundary)) {
-                    $boundary = $boundary * $data['grade'] / 100.0;
-                } else {
+            if (strlen($boundary) > 0) {
+                if ($boundary[strlen($boundary) - 1] == '%') {
+                    $boundary = trim(substr($boundary, 0, -1));
+                    if (is_numeric($boundary)) {
+                        $boundary = $boundary * $data['grade'] / 100.0;
+                    } else {
+                        $errors["feedbackboundaries[$i]"] =
+                                get_string('feedbackerrorboundaryformat', 'quiz', $i + 1);
+                    }
+                } else if (!is_numeric($boundary)) {
                     $errors["feedbackboundaries[$i]"] =
                             get_string('feedbackerrorboundaryformat', 'quiz', $i + 1);
                 }
@@ -511,6 +596,38 @@ class mod_quiz_mod_form extends moodleform_mod {
             }
         }
 
+        // Any other rule plugins.
+        $errors = quiz_access_manager::validate_settings_form_fields($errors, $data, $files, $this);
+
         return $errors;
     }
+
+    /**
+     * Get the maximum number of attempts that anyone might have due to a user
+     * or group override. Used to decide whether disabledIf rules should be applied.
+     * @return int the number of attempts allowed. For the purpose of this method,
+     * unlimited is returned as 1000, not 0.
+     */
+    public function get_max_attempts_for_any_override() {
+        global $DB;
+
+        if (empty($this->_instance)) {
+            // Quiz not created yet, so no overrides.
+            return 1;
+        }
+
+        if ($this->maxattemptsanyoverride === null) {
+            $this->maxattemptsanyoverride = $DB->get_field_sql("
+                    SELECT MAX(CASE WHEN attempts = 0 THEN 1000 ELSE attempts END)
+                      FROM {quiz_overrides}
+                     WHERE quiz = ?",
+                    array($this->_instance));
+            if ($this->maxattemptsanyoverride < 1) {
+                // This happens when no override alters the number of attempts.
+                $this->maxattemptsanyoverride = 1;
+            }
+        }
+
+        return $this->maxattemptsanyoverride;
+    }
 }
