@@ -17,15 +17,28 @@
 /**
  * Lang strings.
  *
- * @package    report
- * @subpackage log
+ * Language strings to be used by report/logs
+ *
+ * @package    report_log
  * @copyright  1999 onwards Martin Dougiamas  {@link http://moodle.com}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
+$string['eventcomponent'] = 'Component';
+$string['eventcontext'] = 'Event context';
+$string['eventloggedas'] = '{$a->realusername} as {$a->asusername}';
+$string['eventorigin'] = 'Origin';
+$string['eventrelatedfullnameuser'] = 'Affected user';
+$string['eventreportviewed'] = 'Log report viewed';
+$string['eventuserreportviewed'] = 'User log report viewed';
 $string['log:view'] = 'View course logs';
 $string['log:viewtoday'] = 'View today\'s logs';
+$string['page'] = 'Page {$a}';
+$string['logsformat'] = 'Logs format';
+$string['mnetlogwarning'] = 'If your MNet clients are running Moodle 2.7 or higher, then you may not receive any logs.';
+$string['nologreaderenabled'] = 'No log reader enabled';
 $string['page-report-log-x'] = 'Any log report';
 $string['page-report-log-index'] = 'Course log report';
 $string['page-report-log-user'] = 'User course log report';
 $string['pluginname'] = 'Logs';
+$string['selectlogreader'] = 'Select log reader';
